@@ -1,13 +1,5 @@
 /*
- *                
- * Filename:      cobex_bfb_private.h
- * Version:       
- * Description:   
- * Status:        Experimental.
- * Author:        Christian W. Zuckschwerdt <zany@triq.net>
- * Created at:    Don, 17 Jan 2002 18:27:25 +0100
- * Modified at:   Don, 17 Jan 2002 23:46:52 +0100
- * Modified by:   Christian W. Zuckschwerdt <zany@triq.net>
+ * cobex_bfb_private.h
  *
  *   Copyright (c) 2002 Christian W. Zuckschwerdt <zany@triq.net>
  * 
@@ -27,16 +19,35 @@
  *     
  */
 
-#include "bfb.h"
+#ifndef COBEXBFB_PRIVATE_H
+#define COBEXBFB_PRIVATE_H
+
+#include <bfb/bfb.h>
 
 #define SERPORT "/dev/ttyS0"
 
+#define	RECVSIZE 500	/* Recieve up to this much from socket */
+
+enum cobex_type
+{
+        CT_SIEMENS,
+        CT_ERICSSON
+};
+
 typedef struct {
-	gchar *tty;
-	int fd;
-	guint8 recv[500];
-	gint recv_len;
-	guint8 seq;
-	bfb_data_t *data;
-	gint data_len;
+	enum cobex_type type;	/* Type of connected mobile */
+	char *tty;
+#ifdef _WIN32
+	HANDLE fd;		/* Socket descriptor */
+#else
+	int fd;			/* Socket descriptor */
+#endif
+	uint8_t recv[RECVSIZE];	/* Buffer socket input */
+	int recv_len;
+	uint8_t seq;
+	bfb_data_t *data_buf;	/* assembled obex frames */
+	int data_size;		/* max buffer size */
+	int data_len;		/* filled buffer length */
 } cobex_t;
+
+#endif /* COBEXBFB_PRIVATE_H */
